@@ -2,110 +2,126 @@ import sys
 import time
 import select
 import microstackcommon.gpio
-from microstackcommon.i2c import I2CMaster, writing_bytes, writing, reading
+import microstackcommon.i2c
 
 
 DEFAULT_I2C_BUS = 1
+DEFAULT_I2C_ADDR_XM = 0x1E
+DEFAULT_I2C_ADDR_G = 0x6A
+
+# Register addresses
+# Gyroscope register addresses
+WHO_AM_I_G = 0x0F
+CTRL_REG1_G = 0x20
+CTRL_REG2_G = 0x21
+CTRL_REG3_G = 0x22
+CTRL_REG4_G = 0x23
+CTRL_REG5_G = 0x24
+REFERENCE_G = 0x25
+STATUS_REG_G = 0x27
+OUT_X_L_G = 0x28
+OUT_X_H_G = 0x29
+OUT_Y_L_G = 0x2A
+OUT_Y_H_G = 0x2B
+OUT_Z_L_G = 0x2C
+OUT_Z_H_G = 0x2D
+FIFO_CTRL_REG_G = 0x2E
+FIFO_SRC_REG_G = 0x2F
+INT1_CFG_G = 0x30
+INT1_SRC_G = 0x31
+INT1_TSH_XH_G = 0x32
+INT1_TSH_XL_G = 0x33
+INT1_TSH_YH_G = 0x34
+INT1_TSH_YL_G = 0x35
+INT1_TSH_ZH_G = 0x36
+INT1_TSH_ZL_G = 0x37
+INT1_DURATION_G = 0x38
+# Accelerometer register addresses
+OUT_TEMP_L_XM = 0x05
+OUT_TEMP_H_XM = 0x06
+STATUS_REG_M = 0x07
+OUT_X_L_M = 0x08
+OUT_X_H_M = 0x09
+OUT_Y_L_M = 0x0A
+OUT_Y_H_M = 0x0B
+OUT_Z_L_M = 0x0C
+OUT_Z_H_M = 0x0D
+WHO_AM_I_XM = 0x0F
+INT_CTRL_REG_M = 0x12
+INT_SRC_REG_M = 0x13
+INT_THS_L_M = 0x14
+INT_THS_H_M = 0x15
+OFFSET_X_L_M = 0x16
+OFFSET_X_H_M = 0x17
+OFFSET_Y_L_M = 0x18
+OFFSET_Y_H_M = 0x19
+OFFSET_Z_L_M = 0x1A
+OFFSET_Z_H_M = 0x1B
+REFERENCE_X = 0x1C
+REFERENCE_Y = 0x1D
+REFERENCE_Z = 0x1E
+CTRL_REG0_XM = 0x1F
+CTRL_REG1_XM = 0x20
+CTRL_REG2_XM = 0x21
+CTRL_REG3_XM = 0x22
+CTRL_REG4_XM = 0x23
+CTRL_REG5_XM = 0x24
+CTRL_REG6_XM = 0x25
+CTRL_REG7_XM = 0x26
+STATUS_REG_A = 0x27
+OUT_X_L_A = 0x28
+OUT_X_H_A = 0x29
+OUT_Y_L_A = 0x2A
+OUT_Y_H_A = 0x2B
+OUT_Z_L_A = 0x2C
+OUT_Z_H_A = 0x2D
+FIFO_CTRL_REG = 0x2E
+FIFO_SRC_REG = 0x2F
+INT_GEN_1_REG = 0x30
+INT_GEN_1_SRC = 0x31
+INT_GEN_1_THS = 0x32
+INT_GEN_1_DURATION = 0x33
+INT_GEN_2_REG = 0x34
+INT_GEN_2_SRC = 0x35
+INT_GEN_2_THS = 0x36
+INT_GEN_2_DURATION = 0x37
+CLICK_CFG = 0x38
+CLICK_SRC = 0x39
+CLICK_THS = 0x3A
+TIME_LIMIT = 0x3B
+TIME_LATENCY = 0x3C
+TIME_WINDOW = 0x3D
+ACT_THS = 0x3E
+ACT_DUR = 0x3F
 
 
-class LSM9DS0RegisterReadOnlyError(Exception):
-    pass
-
-
-class LSM9DS0(I2CMaster):
+class LSM9DS0():
     """iNEMO inertial module: 3D accelerometer, 3D gyroscope, 3D magnetometer.
 
         http://www.st.com/web/en/catalog/sense_power/FM89/SC1448/PF258556
         http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/DM00087365.pdf
 
+    The following attributes are configurable:
+
+               i2c_master - Controls which I2CMaster bus is used to talk
+                            to the device.
+              i2c_reading - Generates I2C read messages for i2c_master.
+        i2c_writing_bytes - Generates I2C write messages for i2c_master.
+              i2c_addr_xm - I2C Address of the accelerometer.
+               i2c_addr_g - I2C Address of the Gyroscope.
     """
 
-    XM_I2C_ADDR = 0x1E
-    G_I2C_ADDR = 0x6A
-
-    def __init__(self, bus=DEFAULT_I2C_BUS):
-        super().__init__(bus)
-        self.who_am_i_g = LSM9DS0Register(0x0F, self.G_I2C_ADDR, self, True)
-        self.ctrl_reg1_g = LSM9DS0Register(0x20, self.G_I2C_ADDR, self)
-        self.ctrl_reg2_g = LSM9DS0Register(0x21, self.G_I2C_ADDR, self)
-        self.ctrl_reg3_g = LSM9DS0Register(0x22, self.G_I2C_ADDR, self)
-        self.ctrl_reg4_g = LSM9DS0Register(0x23, self.G_I2C_ADDR, self)
-        self.ctrl_reg5_g = LSM9DS0Register(0x24, self.G_I2C_ADDR, self)
-        self.reference_g = LSM9DS0Register(0x25, self.G_I2C_ADDR, self)
-        self.status_reg_g = LSM9DS0Register(0x27, self.G_I2C_ADDR, self, True)
-        self.out_x_l_g = LSM9DS0Register(0x28, self.G_I2C_ADDR, self, True)
-        self.out_x_h_g = LSM9DS0Register(0x29, self.G_I2C_ADDR, self, True)
-        self.out_y_l_g = LSM9DS0Register(0x2A, self.G_I2C_ADDR, self, True)
-        self.out_y_h_g = LSM9DS0Register(0x2B, self.G_I2C_ADDR, self, True)
-        self.out_z_l_g = LSM9DS0Register(0x2C, self.G_I2C_ADDR, self, True)
-        self.out_z_h_g = LSM9DS0Register(0x2D, self.G_I2C_ADDR, self, True)
-        self.fifo_ctrl_reg_g = LSM9DS0Register(0x2E, self.G_I2C_ADDR, self)
-        self.fifo_src_reg_g = LSM9DS0Register(0x2F, self.G_I2C_ADDR, self, True)
-        self.int1_cfg_g = LSM9DS0Register(0x30, self.G_I2C_ADDR, self)
-        self.int1_src_g = LSM9DS0Register(0x31, self.G_I2C_ADDR, self, True)
-        self.int1_tsh_xh_g = LSM9DS0Register(0x32, self.G_I2C_ADDR, self)
-        self.int1_tsh_xl_g = LSM9DS0Register(0x33, self.G_I2C_ADDR, self)
-        self.int1_tsh_yh_g = LSM9DS0Register(0x34, self.G_I2C_ADDR, self)
-        self.int1_tsh_yl_g = LSM9DS0Register(0x35, self.G_I2C_ADDR, self)
-        self.int1_tsh_zh_g = LSM9DS0Register(0x36, self.G_I2C_ADDR, self)
-        self.int1_tsh_zl_g = LSM9DS0Register(0x37, self.G_I2C_ADDR, self)
-        self.int1_duration_g = LSM9DS0Register(0x38, self.G_I2C_ADDR, self)
-        self.out_temp_l_xm = LSM9DS0Register(0x05, self.XM_I2C_ADDR, self, True)
-        self.out_temp_h_xm = LSM9DS0Register(0x06, self.XM_I2C_ADDR, self, True)
-        self.status_reg_m = LSM9DS0Register(0x07, self.XM_I2C_ADDR, self, True)
-        self.out_x_l_m = LSM9DS0Register(0x08, self.XM_I2C_ADDR, self, True)
-        self.out_x_h_m = LSM9DS0Register(0x09, self.XM_I2C_ADDR, self, True)
-        self.out_y_l_m = LSM9DS0Register(0x0A, self.XM_I2C_ADDR, self, True)
-        self.out_y_h_m = LSM9DS0Register(0x0B, self.XM_I2C_ADDR, self, True)
-        self.out_z_l_m = LSM9DS0Register(0x0C, self.XM_I2C_ADDR, self, True)
-        self.out_z_h_m = LSM9DS0Register(0x0D, self.XM_I2C_ADDR, self, True)
-        self.who_am_i_xm = LSM9DS0Register(0x0F, self.XM_I2C_ADDR, self, True)
-        self.int_ctrl_reg_m = LSM9DS0Register(0x12, self.XM_I2C_ADDR, self)
-        self.int_src_reg_m = LSM9DS0Register(0x13, self.XM_I2C_ADDR, self, True)
-        self.int_ths_l_m = LSM9DS0Register(0x14, self.XM_I2C_ADDR, self)
-        self.int_ths_h_m = LSM9DS0Register(0x15, self.XM_I2C_ADDR, self)
-        self.offset_x_l_m = LSM9DS0Register(0x16, self.XM_I2C_ADDR, self)
-        self.offset_x_h_m = LSM9DS0Register(0x17, self.XM_I2C_ADDR, self)
-        self.offset_y_l_m = LSM9DS0Register(0x18, self.XM_I2C_ADDR, self)
-        self.offset_y_h_m = LSM9DS0Register(0x19, self.XM_I2C_ADDR, self)
-        self.offset_z_l_m = LSM9DS0Register(0x1A, self.XM_I2C_ADDR, self)
-        self.offset_z_h_m = LSM9DS0Register(0x1B, self.XM_I2C_ADDR, self)
-        self.reference_x = LSM9DS0Register(0x1C, self.XM_I2C_ADDR, self)
-        self.reference_y = LSM9DS0Register(0x1D, self.XM_I2C_ADDR, self)
-        self.reference_z = LSM9DS0Register(0x1E, self.XM_I2C_ADDR, self)
-        self.ctrl_reg0_xm = LSM9DS0Register(0x1F, self.XM_I2C_ADDR, self)
-        self.ctrl_reg1_xm = LSM9DS0Register(0x20, self.XM_I2C_ADDR, self)
-        self.ctrl_reg2_xm = LSM9DS0Register(0x21, self.XM_I2C_ADDR, self)
-        self.ctrl_reg3_xm = LSM9DS0Register(0x22, self.XM_I2C_ADDR, self)
-        self.ctrl_reg4_xm = LSM9DS0Register(0x23, self.XM_I2C_ADDR, self)
-        self.ctrl_reg5_xm = LSM9DS0Register(0x24, self.XM_I2C_ADDR, self)
-        self.ctrl_reg6_xm = LSM9DS0Register(0x25, self.XM_I2C_ADDR, self)
-        self.ctrl_reg7_xm = LSM9DS0Register(0x26, self.XM_I2C_ADDR, self)
-        self.status_reg_a = LSM9DS0Register(0x27, self.XM_I2C_ADDR, self, True)
-        self.out_x_l_a = LSM9DS0Register(0x28, self.XM_I2C_ADDR, self, True)
-        self.out_x_h_a = LSM9DS0Register(0x29, self.XM_I2C_ADDR, self, True)
-        self.out_y_l_a = LSM9DS0Register(0x2A, self.XM_I2C_ADDR, self, True)
-        self.out_y_h_a = LSM9DS0Register(0x2B, self.XM_I2C_ADDR, self, True)
-        self.out_z_l_a = LSM9DS0Register(0x2C, self.XM_I2C_ADDR, self, True)
-        self.out_z_h_a = LSM9DS0Register(0x2D, self.XM_I2C_ADDR, self, True)
-        self.fifo_ctrl_reg = LSM9DS0Register(0x2E, self.XM_I2C_ADDR, self)
-        self.fifo_src_reg = LSM9DS0Register(0x2F, self.XM_I2C_ADDR, self, True)
-        self.int_gen_1_reg = LSM9DS0Register(0x30, self.XM_I2C_ADDR, self)
-        self.int_gen_1_src = LSM9DS0Register(0x31, self.XM_I2C_ADDR, self, True)
-        self.int_gen_1_ths = LSM9DS0Register(0x32, self.XM_I2C_ADDR, self)
-        self.int_gen_1_duration = LSM9DS0Register(0x33, self.XM_I2C_ADDR, self)
-        self.int_gen_2_reg = LSM9DS0Register(0x34, self.XM_I2C_ADDR, self)
-        self.int_gen_2_src = LSM9DS0Register(0x35, self.XM_I2C_ADDR, self, True)
-        self.int_gen_2_ths = LSM9DS0Register(0x36, self.XM_I2C_ADDR, self)
-        self.int_gen_2_duration = LSM9DS0Register(0x37, self.XM_I2C_ADDR, self)
-        self.click_cfg = LSM9DS0Register(0x38, self.XM_I2C_ADDR, self)
-        self.click_src = LSM9DS0Register(0x39, self.XM_I2C_ADDR, self, True)
-        self.click_ths = LSM9DS0Register(0x3A, self.XM_I2C_ADDR, self)
-        self.time_limit = LSM9DS0Register(0x3B, self.XM_I2C_ADDR, self)
-        self.time_latency = LSM9DS0Register(0x3C, self.XM_I2C_ADDR, self)
-        self.time_window = LSM9DS0Register(0x3D, self.XM_I2C_ADDR, self)
-        self.act_ths = LSM9DS0Register(0x3E, self.XM_I2C_ADDR, self)
-        self.act_dur = LSM9DS0Register(0x3F, self.XM_I2C_ADDR, self)
+    def __init__(self,
+                 i2c_master=microstackcommon.i2c.I2CMaster(DEFAULT_I2C_BUS),
+                 i2c_reading=microstackcommon.i2c.reading,
+                 i2c_writing_bytes=microstackcommon.i2c.writing_bytes,
+                 i2c_addr_xm=DEFAULT_I2C_ADDR_XM,
+                 i2c_addr_g=DEFAULT_I2C_ADDR_G):
+        self.i2c_master = i2c_master
+        self.i2c_reading = i2c_reading
+        self.i2c_writing_bytes = i2c_writing_bytes
+        self.i2c_addr_xm = i2c_addr_xm
+        self.i2c_addr_g = i2c_addr_g
 
         self.int_xm_pin = microstackcommon.gpio.Pin(
             27,
@@ -120,7 +136,7 @@ class LSM9DS0(I2CMaster):
         self.int_g_pin_epoll = select.epoll()
 
     def __enter__(self):
-        self = super().__enter__()
+        self.i2c_master = self.i2c_master.__enter__()
         self.verify_whoami()
         # interrupts TODO do this in the set method for each thing
         # self.int_xm_pin.open()
@@ -130,25 +146,64 @@ class LSM9DS0(I2CMaster):
         # self.int_xm_pin_epoll.poll(timeout=0.001)
         return self
 
+    def __exit__(self, *args):
+        self.i2c_master.__exit__(*args)
+
+    def get(self, device_address, register_address):
+        return self.i2c_master.transaction(
+            self.i2c_writing_bytes(device_address, register_address),
+            self.i2c_reading(device_address, 1))[0][0]
+
+    def get_xm(self, register_address):
+        return self.get(self.i2c_addr_xm, register_address)
+
+    def get_g(self, register_address):
+        return self.get(self.i2c_addr_g, register_address)
+
+    def get_bulk(self, device_address, register_address, num_registers):
+        """Returns many registers starting from (and including) this one.
+        Returns registers in byte format, access using indexes.
+        """
+        RAAI = 0x80  # Register Address Auto Increment
+        return self.i2c_master.transaction(
+            self.i2c_writing_bytes(device_address, RAAI | register_address),
+            self.i2c_reading(device_address, num_registers))[0]
+
+    def get_bulk_xm(self, register_address, num_registers):
+        return self.get_bulk(self.i2c_addr_xm, register_address, num_registers)
+
+    def get_bulk_g(self, register_address, num_registers):
+        return self.get_bulk(self.i2c_addr_g, register_address, num_registers)
+
+    def set(self, device_address, register_address, v):
+        self.i2c_master.transaction(
+            self.i2c_writing_bytes(device_address, register_address, v))
+
+    def set_xm(self, register_address, v):
+        self.set(self.i2c_addr_xm, register_address, v)
+
+    def set_g(self, register_address, v):
+        self.set(self.i2c_addr_g, register_address, v)
+
     def verify_whoami(self):
-        assert(self.who_am_i_xm.get() == 0x49)
-        assert(self.who_am_i_g.get() == 0xD4)
+        assert(self.get_xm(WHO_AM_I_XM) == 0x49)
+        assert(self.get_g(WHO_AM_I_G) == 0xD4)
 
     def enable_temperature(self):
-        self.ctrl_reg5_xm.set(self.ctrl_reg5_xm.get() | 0x80)
+        self.set_xm(CTRL_REG5_XM, self.get_xm(CTRL_REG5_XM) | 0x80)
 
     def get_temperature(self):
-        t_low, t_high = self.out_temp_l_xm.get_bulk(2)
+        t_low, t_high = self.get_bulk_xm(OUT_TEMP_L_XM, 2)
         return twos_complement((t_high << 8) | t_low, 12)
 
     def setup_interrupts(self):
         # interrupts
         # CTRL_REG3
         # ctrl_reg3_value = 0
-        # self.ctrl_reg3_xm.set(ctrl_reg3_value)
+        # self.set_xm(CTRL_REG3_XM, ctrl_reg3_value)
         # CTRL_REG4
         # ctrl_reg4_value = 0
-        # self.ctrl_reg4_xm.set(ctrl_reg4_value)
+        # self.set_xm(CTRL_REG4_XM, ctrl_reg4_value)
         raise NotImplementedError()
 
     def setup_accelerometer(self,
@@ -192,7 +247,7 @@ class LSM9DS0(I2CMaster):
             ctrl_reg0_value |= 1 << 5
         if high_pass_filter_enable:
             ctrl_reg0_value |= 1 << 2
-        self.ctrl_reg0_xm.set(ctrl_reg0_value)
+        self.set_xm(CTRL_REG0_XM, ctrl_reg0_value)
 
         # CTRL_REG1
         ctrl_reg1_value = 0
@@ -217,7 +272,7 @@ class LSM9DS0(I2CMaster):
             ctrl_reg1_value |= 1 << 1
         if x_enable:
             ctrl_reg1_value |= 1
-        self.ctrl_reg1_xm.set(ctrl_reg1_value)
+        self.set_xm(CTRL_REG1_XM, ctrl_reg1_value)
 
         # CTRL_REG2
         ctrl_reg2_value = 0
@@ -240,7 +295,7 @@ class LSM9DS0(I2CMaster):
               'negative': 0b10 << 1}
         if self_test in st:
             ctrl_reg2_value |= st[self_test]
-        self.ctrl_reg2_xm.set(ctrl_reg2_value)
+        self.set_xm(CTRL_REG2_XM, ctrl_reg2_value)
 
         # CTRL_REG3
         ctrl_reg3_value = 0
@@ -257,8 +312,8 @@ class LSM9DS0(I2CMaster):
         if fifo_empty_on_int1_xm:
             ctrl_reg3_value |= 1
         # do not disturb the magnetometer values
-        ctrl_reg3_value |= self.ctrl_reg3_xm.get() & 0x0A
-        self.ctrl_reg3_xm.set(ctrl_reg3_value)
+        ctrl_reg3_value |= self.get_xm(CTRL_REG3_XM) & 0x0A
+        self.set_xm(CTRL_REG3_XM, ctrl_reg3_value)
 
         # CTRL_REG4
         # int2_xm config -- not wired in
@@ -276,8 +331,8 @@ class LSM9DS0(I2CMaster):
         # if fifo_watermark_on_int2_xm:
         #     ctrl_reg4_value |= 1
         # # do not disturb the magnetometer values
-        # ctrl_reg4_value |= self.ctrl_reg4_xm.get() & 0x14
-        # self.ctrl_reg4_xm.set(ctrl_reg4_value)
+        # ctrl_reg4_value |= self.get_xm(CTRL_REG4_XM) & 0x14
+        # self.set_xm(CTRL_REG4_XM, ctrl_reg4_value)
 
         # CTRL_REG5
         ctrl_reg5_value = 0
@@ -286,8 +341,8 @@ class LSM9DS0(I2CMaster):
         if latch_interrupt_request_1:
             ctrl_reg5_value |= 1
         # do not disturb the magnetometer values
-        ctrl_reg5_value |= self.ctrl_reg5_xm.get() & 0xFC
-        self.ctrl_reg5_xm.set(ctrl_reg5_value)
+        ctrl_reg5_value |= self.get_xm(CTRL_REG5_XM) & 0xFC
+        self.set_xm(CTRL_REG5_XM, ctrl_reg5_value)
 
         # CTRL_REG7
         ctrl_reg7_value = 0
@@ -300,8 +355,8 @@ class LSM9DS0(I2CMaster):
         if filtered_data_selection:
             ctrl_reg7_value |= 1 << 5
         # do not disturb other half of CTRL_REG7
-        ctrl_reg7_value |= self.ctrl_reg7_xm.get() & 0x1F
-        self.ctrl_reg7_xm.set(ctrl_reg7_value)
+        ctrl_reg7_value |= self.get_xm(CTRL_REG7_XM) & 0x1F
+        self.set_xm(CTRL_REG7_XM, ctrl_reg7_value)
 
         if interrupt_enable:
             int_gen_2_reg_value = 0
@@ -321,12 +376,12 @@ class LSM9DS0(I2CMaster):
                 int_gen_2_reg_value |= 1 << 1
             if x_low_interrupt_enable:
                 int_gen_2_reg_value |= 1
-            self.int_gen_2_reg.set(int_gen_2_reg_value)
+            self.set_xm(INT_GEN_2_REG, int_gen_2_reg_value)
 
-            self.int_gen_1_duration.set(0x7f & interrupt_duration)
+            self.set_xm(INT_GEN_1_DURATION, 0x7f & interrupt_duration)
             interrupt_threshold_raw = int(
                 interrupt_threshold / self._accelerometer_full_scale * 0x7f)
-            self.int_gen_1_ths.set(0x7f & interrupt_threshold_raw)
+            self.set_xm(INT_GEN_1_THS, 0x7f & interrupt_threshold_raw)
 
             if not self.int_xm_pin.closed:
                 self.int_xm_pin.open()
@@ -372,10 +427,10 @@ class LSM9DS0(I2CMaster):
                 int_ctrl_reg_m_value |= 1 << 2
             if enable_4d:
                 int_ctrl_reg_m_value |= 1 << 1
-            self.int_ctrl_reg_m.set(int_ctrl_reg_m_value)
+            self.set_xm(INT_CTRL_REG_M, int_ctrl_reg_m_value)
 
-            self.int_ths_h_m.set(0xff & (interrupt_threshold >> 8))
-            self.int_ths_l_m.set(0xff & interrupt_threshold)
+            self.set_xm(INT_THS_H_M, 0xff & (interrupt_threshold >> 8))
+            self.set_xm(INT_THS_L_M, 0xff & interrupt_threshold)
 
             if not self.int_xm_pin.closed:
                 self.int_xm_pin.open()
@@ -402,7 +457,7 @@ class LSM9DS0(I2CMaster):
             ctrl_reg5_value |= 1 << 1
         if latch_interrupt_request_1:
             ctrl_reg5_value |= 1
-        self.ctrl_reg5_xm.set(ctrl_reg5_value)
+        self.set_xm(CTRL_REG5_XM, ctrl_reg5_value)
 
         # CTRL_REG6
         ctrl_reg6_value = 0
@@ -413,7 +468,7 @@ class LSM9DS0(I2CMaster):
         if full_scale in full_scales:
             ctrl_reg6_value |= full_scales[full_scale]
             self._magnetometer_full_scale = full_scale
-        self.ctrl_reg6_xm.set(ctrl_reg6_value)
+        self.set_xm(CTRL_REG6_XM, ctrl_reg6_value)
 
         # CTRL_REG7
         ctrl_reg7_value = 0
@@ -425,8 +480,8 @@ class LSM9DS0(I2CMaster):
         if sensor_mode in sensor_modes:
             ctrl_reg7_value |= sensor_modes[sensor_mode]
         # do not disturb other half of CTRL_REG7
-        ctrl_reg7_value |= self.ctrl_reg7_xm.get() & 0xF8
-        self.ctrl_reg7_xm.set(ctrl_reg7_value)
+        ctrl_reg7_value |= self.get_xm(CTRL_REG7_XM) & 0xF8
+        self.set_xm(CTRL_REG7_XM, ctrl_reg7_value)
 
     def setup_gyroscope(self,
                         data_rate=95,
@@ -482,17 +537,17 @@ class LSM9DS0(I2CMaster):
                 int1_cfg_g_value |= 1 << 1
             if x_low_interrupt_enable:
                 int1_cfg_g_value |= 1
-            self.int1_cfg_g.set(int1_cfg_g_value)
+            self.set_g(INT1_CFG_G, int1_cfg_g_value)
 
-            self.int1_tsh_xh_g.set(0xff & (x_interrupt_threshold >> 8))
-            self.int1_tsh_xl_g.set(0xff & x_interrupt_threshold)
-            self.int1_tsh_yh_g.set(0xff & (y_interrupt_threshold >> 8))
-            self.int1_tsh_yl_g.set(0xff & y_interrupt_threshold)
-            self.int1_tsh_zh_g.set(0xff & (z_interrupt_threshold >> 8))
-            self.int1_tsh_zl_g.set(0xff & z_interrupt_threshold)
+            self.set_g(INT1_TSH_XH_G, 0xff & (x_interrupt_threshold >> 8))
+            self.set_g(INT1_TSH_XL_G, 0xff & x_interrupt_threshold)
+            self.set_g(INT1_TSH_YH_G, 0xff & (y_interrupt_threshold >> 8))
+            self.set_g(INT1_TSH_YL_G, 0xff & y_interrupt_threshold)
+            self.set_g(INT1_TSH_ZH_G, 0xff & (z_interrupt_threshold >> 8))
+            self.set_g(INT1_TSH_ZL_G, 0xff & z_interrupt_threshold)
 
             if interrupt_duration:
-                self.int1_duration_g.set(0x80 | (0x7f & interrupt_duration))
+                self.set_g(INT1_DURATION_G, 0x80 | (0x7f & interrupt_duration))
 
             self.int_g_pin.open()
             self.int_g_pin_epoll.register(self.int_g_pin,
@@ -518,7 +573,7 @@ class LSM9DS0(I2CMaster):
             ctrl_reg1_value |= 1 << 1
         if x_enable:
             ctrl_reg1_value |= 1
-        self.ctrl_reg1_g.set(ctrl_reg1_value)
+        self.set_g(CTRL_REG1_G, ctrl_reg1_value)
 
         # CTRL_REG2
         ctrl_reg2_value = 0
@@ -530,7 +585,7 @@ class LSM9DS0(I2CMaster):
             ctrl_reg2_value |= high_pass_filter_modes[high_pass_filter_mode]
         if high_pass_filter_cutoff_selection in range(10):
             ctrl_reg2_value |= high_pass_filter_cutoff_selection
-        self.ctrl_reg2_g.set(ctrl_reg2_value)
+        self.set_g(CTRL_REG2_G, ctrl_reg2_value)
 
         # CTRL_REG3
         ctrl_reg3_value = 0
@@ -543,7 +598,7 @@ class LSM9DS0(I2CMaster):
         if open_drain:
             ctrl_reg3_value |= 1 << 4
         # lower nibble is to do with watermark on DRDY which is not connected.
-        self.ctrl_reg3_g.set(ctrl_reg3_value)
+        self.set_g(CTRL_REG3_G, ctrl_reg3_value)
 
         # CTRL_REG4
         ctrl_reg4_value = 0
@@ -563,7 +618,7 @@ class LSM9DS0(I2CMaster):
         if self_test_mode in self_test_modes:
             ctrl_reg4_value |= self_test_modes[self_test_mode]
         # SPI mode (bit 0) is obviously disabled since this is an I2C module
-        self.ctrl_reg4_g.set(ctrl_reg4_value)
+        self.set_g(CTRL_REG4_G, ctrl_reg4_value)
 
         # CTRL_REG5
         ctrl_reg5_value = 0
@@ -577,10 +632,10 @@ class LSM9DS0(I2CMaster):
             ctrl_reg5_value |= int1_selection << 2
         if out_selection in range(4):
             ctrl_reg5_value |= out_selection
-        self.ctrl_reg5_g.set(ctrl_reg5_value)
+        self.set_g(CTRL_REG5_G, ctrl_reg5_value)
 
         # DEBUG print out all control registers
-        # ctrlregs = self.ctrl_reg1_g.get_bulk(5)
+        # ctrlregs = self.get_bulk(CTRL_REG1_G, 5)
         # for i in range(5):
         #     print("CTRL_REG{}_G: {}".format(i+1, hex(ctrlregs[i])))
 
@@ -594,10 +649,10 @@ class LSM9DS0(I2CMaster):
             fifo_ctrl_reg_g_value |= fifo_modes[fifo_mode]
         if fifo_watermark in range(32):
             fifo_ctrl_reg_g_value |= fifo_watermark
-        self.fifo_ctrl_reg_g.set(fifo_ctrl_reg_g_value)
+        self.set_g(FIFO_CTRL_REG_G, fifo_ctrl_reg_g_value)
 
-    def _get_twos_complement_xyz(self, register):
-        xlo, xhi, ylo, yhi, zlo, zhi = register.get_bulk(6)
+    def _get_twos_complement_xyz(self, start_register_addr):
+        xlo, xhi, ylo, yhi, zlo, zhi = self.get_bulk_xm(start_register_addr, 6)
         v = {'x': xhi << 8 | xlo, 'y': yhi << 8 | ylo, 'z': zhi << 8 | zlo}
         return {axis: twos_complement(magnitude, 16)
                 for axis, magnitude in v.items()}
@@ -606,7 +661,7 @@ class LSM9DS0(I2CMaster):
         return magnitude / 2**15 * scale
 
     def get_accelerometer(self, raw=False):
-        acceleration_vector = self._get_twos_complement_xyz(self.out_x_l_a)
+        acceleration_vector = self._get_twos_complement_xyz(OUT_X_L_A)
         if raw:
             return acceleration_vector
         else:
@@ -615,7 +670,7 @@ class LSM9DS0(I2CMaster):
                     for axis, magnitude in acceleration_vector.items()}
 
     def get_magnetometer(self, raw=False):
-        magnetic_vector = self._get_twos_complement_xyz(self.out_x_l_m)
+        magnetic_vector = self._get_twos_complement_xyz(OUT_X_L_M)
         if raw:
             return magnetic_vector
         else:
@@ -623,7 +678,7 @@ class LSM9DS0(I2CMaster):
                     for axis, magnitude in magnetic_vector.items()}
 
     def get_gyroscope(self, raw=False):
-        gyro_vector = self._get_twos_complement_xyz(self.out_x_l_g)
+        gyro_vector = self._get_twos_complement_xyz(OUT_X_L_G)
         if raw:
             return gyro_vector
         else:
@@ -639,7 +694,7 @@ class LSM9DS0(I2CMaster):
 
             for fileno, event in events:
                 if fileno == self.int_g_pin.fileno():
-                    int_src = self.int1_src_g.get()
+                    int_src = self.get_g(INT1_SRC_G)
                     return {'active': (1 & (int_src >> 6)) == 1,
                             'z_high': (1 & (int_src >> 5)) == 1,
                             'z_low': (1 & (int_src >> 4)) == 1,
@@ -686,42 +741,6 @@ class LSM9DS0(I2CMaster):
                             'active': (1 & int_src) == 1}
 
 
-class LSM9DS0Register(object):
-
-    def __init__(self,
-                 register_address,
-                 device_address,
-                 i2c_master,
-                 read_only=False):
-        self.register_address = register_address
-        self.device_address = device_address
-        self.i2c_master = i2c_master
-        self.read_only = read_only
-
-    def get(self):
-        return self.i2c_master.transaction(
-            writing_bytes(self.device_address, self.register_address),
-            reading(self.device_address, 1))[0][0]
-
-    def get_bulk(self, num_registers):
-        """Returns many registers starting from (and including) this one.
-        Returns registers in byte format, access using indexes.
-        """
-        RAAI = 0x80  # Register Address Auto Increment
-        return self.i2c_master.transaction(
-            writing_bytes(self.device_address, RAAI | self.register_address),
-            reading(self.device_address, num_registers))[0]
-
-    def set(self, v):
-        if self.read_only:
-            raise LSM9DS0RegisterReadOnlyError(
-                "Register at address {} is read only".format(
-                    hex(register_address)))
-        else:
-            self.i2c_master.transaction(
-                writing_bytes(self.device_address, self.register_address, v))
-
-
 def twos_complement(value, bits):
     """Signs a value with an arbitary number of bits."""
     if value >= (1 << (bits - 1)):
